@@ -20,7 +20,6 @@ from .sampler import Sobol, Lhs, Hammersly, Halton, Grid
 from .sampler import InitialPointGenerator
 from .space import Space, Categorical, Integer, Real, Dimension
 
-
 __all__ = (
     "load",
     "dump",
@@ -203,9 +202,8 @@ def check_x_in_space(x, space):
 
 
 def expected_minimum(res, n_random_starts=20, random_state=None):
-    """
-    Compute the minimum over the predictions of the last surrogate model.
-    Uses `expected_minimum_random_sampling` with `n_random_starts`=100000,
+    """Compute the minimum over the predictions of the last surrogate model.
+    Uses `expected_minimum_random_sampling` with `n_random_starts` = 100000,
     when the space contains any categorical values.
 
     .. note::
@@ -334,8 +332,7 @@ def has_gradients(estimator):
 
 
 def cook_estimator(base_estimator, space=None, **kwargs):
-    """
-    Cook a default estimator.
+    """Cook a default estimator.
 
     For the special base_estimator called "DUMMY" the return value is None.
     This corresponds to sampling points at random, hence there is no need
@@ -343,8 +340,7 @@ def cook_estimator(base_estimator, space=None, **kwargs):
 
     Parameters
     ----------
-    base_estimator : "GP", "RF", "ET", "GBRT", "DUMMY"
-                        or sklearn regressor
+    base_estimator : "GP", "RF", "ET", "GBRT", "DUMMY" or sklearn regressor
         Should inherit from `sklearn.base.RegressorMixin`.
         In addition the `predict` method should have an optional `return_std`
         argument, which returns `std(Y | x)`` along with `E[Y | x]`.
@@ -374,7 +370,6 @@ def cook_estimator(base_estimator, space=None, **kwargs):
             space = Space(normalize_dimensions(space.dimensions))
             n_dims = space.transformed_n_dims
             is_cat = space.is_categorical
-
         else:
             raise ValueError("Expected a Space instance, not None.")
 
@@ -404,20 +399,22 @@ def cook_estimator(base_estimator, space=None, **kwargs):
     elif base_estimator == "DUMMY":
         return None
 
+    if ('n_jobs' in kwargs.keys()) and not hasattr(base_estimator, 'n_jobs'):
+        del kwargs['n_jobs']
+
     base_estimator.set_params(**kwargs)
     return base_estimator
 
 
 def cook_initial_point_generator(generator, **kwargs):
-    """
-    Cook a default initial point generator.
+    """Cook a default initial point generator.
 
     For the special generator called "random" the return value is None.
 
     Parameters
     ----------
-    generator : "lhs", "sobol", "halton", "hammersly", "grid", "random"
-                        or InitialPointGenerator instance"
+    generator : "lhs", "sobol", "halton", "hammersly", "grid", "random" \
+            or InitialPointGenerator instance"
         Should inherit from `skopt.sampler.InitialPointGenerator`.
 
     kwargs : dict
@@ -595,37 +592,18 @@ def normalize_dimensions(dimensions):
     """
     space = Space(dimensions)
     transformed_dimensions = []
-    if space.is_categorical:
-        # recreate the space and explicitly set transform to "string"
-        # this is a special case for GP based regressors
-        for dimension in space:
-            transformed_dimensions.append(Categorical(dimension.categories,
-                                                      dimension.prior,
-                                                      name=dimension.name,
-                                                      transform="string"))
-
-    else:
-        for dimension in space.dimensions:
-            if isinstance(dimension, Categorical):
-                transformed_dimensions.append(dimension)
-            # To make sure that GP operates in the [0, 1] space
-            elif isinstance(dimension, Real):
-                transformed_dimensions.append(
-                    Real(dimension.low, dimension.high, dimension.prior,
-                         name=dimension.name,
-                         transform="normalize",
-                         dtype=dimension.dtype)
-                    )
-            elif isinstance(dimension, Integer):
-                transformed_dimensions.append(
-                    Integer(dimension.low, dimension.high,
-                            name=dimension.name,
-                            transform="normalize",
-                            dtype=dimension.dtype)
-                    )
-            else:
-                raise RuntimeError("Unknown dimension type "
-                                   "(%s)" % type(dimension))
+    for dimension in space.dimensions:
+        # check if dimension is of a Dimension instance
+        if isinstance(dimension, Dimension):
+            # Change the transformer to normalize
+            # and add it to the new transformed dimensions
+            dimension.set_transformer("normalize")
+            transformed_dimensions.append(
+                dimension
+            )
+        else:
+            raise RuntimeError("Unknown dimension type "
+                               "(%s)" % type(dimension))
 
     return Space(transformed_dimensions)
 
@@ -736,10 +714,10 @@ def use_named_args(dimensions):
     ...                          n_calls=20, base_estimator="ET",
     ...                          random_state=4)
     >>>
-    >>> # Print the best-found results.
-    >>> print("Best fitness:", result.fun)
+    >>> # Print the best-found results in same format as the expected result.
+    >>> print("Best fitness: " + str(result.fun))
     Best fitness: 0.1948080835239698
-    >>> print("Best parameters:", result.x)
+    >>> print("Best parameters: {}".format(result.x))
     Best parameters: [0.44134853091052617, 0.06570954323368307, 0.17586123323419825]
 
     Parameters
